from openwisp_users.tests.test_admin import TestBasicUsersIntegration


class TestUsersIntegration(TestBasicUsersIntegration):
    """
    tests integration with openwisp_users
    """

    pass


del TestBasicUsersIntegration
