<!DOCTYPE html>{% load static %}
<html>
<head>
    <title>{% block title %}{% endblock %}</title>
    <meta charset="utf-8">
    <script src = "{% static 'admin/js/vendor/jquery/jquery.js' %}"></script>
    <script src = "{% static 'netjsongraph/js/lib/jquery-ui.min.js' %}"></script>
    {% block style %}
        <link href="{% static 'netjsongraph/css/src/netjsongraph.css' %}" rel="stylesheet">
        <link href="{% static 'netjsongraph/css/lib/jquery-ui.min.css' %}" rel="stylesheet">
        <link href="{% static VISUALIZER_CSS %}" rel="stylesheet">
    {% endblock %}
</head>
<body class="frontend">
    {% block body %}{% endblock %}
    {% block script %}
        <script src="{% static 'netjsongraph/js/lib/d3.min.js' %}"></script>
        <script src="{% static 'netjsongraph/js/src/netjsongraph.js' %}"></script>
        <script src="{% static 'netjsongraph/js/topology-history.js' %}"></script>
    {% endblock %}
</body>
</html>
