# Generated by Django 3.0.7 on 2020-06-22 16:24

import collections

import jsonfield.fields
import rest_framework.utils.encoders
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('topology', '0009_remove_old_addresses'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='properties',
            field=jsonfield.fields.JSONField(
                blank=True,
                default=dict,
                dump_kwargs={
                    'cls': rest_framework.utils.encoders.JSONEncoder,
                    'indent': 4,
                },
                load_kwargs={'object_pairs_hook': collections.OrderedDict},
            ),
        ),
        migrations.AlterField(
            model_name='node',
            name='addresses',
            field=jsonfield.fields.JSONField(blank=True, default=[]),
        ),
        migrations.AlterField(
            model_name='node',
            name='properties',
            field=jsonfield.fields.JSONField(
                blank=True,
                default=dict,
                dump_kwargs={
                    'cls': rest_framework.utils.encoders.JSONEncoder,
                    'indent': 4,
                },
                load_kwargs={'object_pairs_hook': collections.OrderedDict},
            ),
        ),
    ]
