input[type=text].readonly {
    border: 1px solid rgba(0, 0, 0, 0.05) !important;
    background-color: rgba(0, 0, 0, 0.070);
}

#id_receive_url {
    width: 435px
}

.djnjg-overlay {
    display: none;
    position: fixed;
    left: 0;
    top: 0;
    z-index: 11;
    width: 100%;
    height: 100%;
    background: rgba(0, 0, 0, 0.95);
}

.djnjg-overlay .inner {
    width: 100%;
    height: 100%;
    overflow: auto;
    padding: 0;
}

.djnjg-overlay .close,
.djnjg-overlay .close:focus,
.djnjg-overlay .close:active {
    position: absolute;
    right: 10px;
    top: 10px;
    font-weight: bold;
    font-size: 15px;
    background-color: #772299;
    color: #fff;
    padding: 5px 8px;
    border-radius: 5px;
    text-decoration: none;
}

.djnjg-overlay .close:hover {
    background-color: #9e47c1;
}

.njg-overlay {
    top: auto;
    bottom: 60px;
    right: 5px;
}
