{% load i18n %}
<div id="legend">
    <p><span class="link up">&nbsp;</span> {% trans 'link up' %}</p>
    <p><span class="link down">&nbsp;</span> {% trans 'link down' %}</p>
</div>
<div class="njg-switcher">
    <label for="njg-datepicker">{% trans 'date' %}:</label>
    <input type="text" id="njg-datepicker"
           data-history-api="{{ history_url }}">
</div>
